=begin
Copyright 2009, Matt666
All Rights Reserved

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

License:   		Free

Author:     	Matthieu NOBLET
Organization:   Matt666
Name:      		Normalize
Version:   		1.00
SU Version:   	6 til 8
Date:      		25/10/2009
Description:   	Puts all entities on layer 0
Usage:      	Type "Matt_Layers::Normalize.selection" in the ruby console for a selection.
				the whole model.
				Type "Matt_Layers::Normalize.wd" in the ruby console for changing filter options.

History:
	1.000		20091025
				First version
=end

module Matt_Layers

module Normalize
module_function

def selection
	puts 'Matt_Layers::Normalize.selection'
	model=Sketchup.active_model
	sel=model.selection
	return if sel.empty?
	msg='Normalize selection'
	Sketchup.version.to_i>=7 ? model.start_operation(msg,true) : model.start_operation(msg)
	@n=0
	@flag=false
	ents=[]
	l0 = find_layer_zero
	filter=filter_list
	sel.each do |e|
		boucle e,l0,ents,filter
	end
	puts "#{@n}entitie(s) normalized."
	model.commit_operation
end

def all
	puts 'Matt_Layers::Normalize.all'
	model=Sketchup.active_model
	msg='Normalize model'
	Sketchup.version.to_i>=7 ? model.start_operation(msg,true) : model.start_operation(msg)
	@n=0
	@flag=false
	ents=[]
	l0=find_layer_zero
	filter=filter_list
	model.entities.each do |e|
		boucle e,l0,ents,filter
	end
	insts=ents.select{|e|e.is_a?(Sketchup::ComponentInstance) and e.is_a?(Sketchup::Group)}
	for d in model.definitions
		boucle d,l0,ents,filter
		for i in d.instances
			boucle i,l0,ents,filter if not(insts.include?(i))
		end
	end
	puts "#{@n} entitie(s) normalized."
	model.commit_operation
end

def boucle(ent,l0,lst,flt)
	if flt.include?(ent.class) and ent.layer!=l0
		ent.layer=nil
		@n+=1
	end
	
	lst.push ent
	if @flag==true
		if ent.is_a? Sketchup::Group
			ent.entities.each{|e| boucle e,l0,lst,flt}
		elsif ent.is_a? Sketchup::ComponentInstance
			ent.definition.entities.each{|e| boucle e,l0,lst,flt}
		end
	end
end

def find_layer_zero
	e=Sketchup.active_model.active_entities[0]
	entity_layer=e.layer
	e.layer=nil
	l0=e.layer
	e.layer=entity_layer
	return l0
end

def wd
	puts 'Matt_Layers::Normalize.wd'
	@dir=File.dirname(__FILE__)
	settings=IO.readlines(File.join(@dir,'Matt_Normalize/default.txt'))
	settings.each{|l|l.strip!}
	initial_settings=settings.dup
	
	dial=UI::WebDialog.new('Matt_Normalize', false, 'Matt_Normalize', 315, 400, 150, 100, false)
	dial.set_file(File.join(@dir, "Matt_Normalize/normalize_manager.html"))
	
	nb=0
	if RUBY_PLATFORM.include?('darwin')
		dial.show_modal{
			for e in settings
				str="rubyCheck('c#{nb}','#{e}')"
				dial.execute_script(str)
				nb+=1
			end
		}
	else
		dial.show{
			for e in settings
				str="rubyCheck('c#{nb}','#{e}')"
				dial.execute_script(str)
				nb+=1
			end
		}
	end
	
	dial.add_action_callback("cback") {|dialog, params|
		case params
		when "stop"
			dial.close
		when "go"
			if settings!=initial_settings
				f=File.open(File.join(@dir,'Matt_Normalize/default.txt'),"w")
				for e in settings
					f.puts e
				end
				f.close
			end
			dial.close
		when "0","1","2","3","4","5","6","7","8","9"
			nb=params.to_i
			if settings[nb]=="true"
				settings[nb]="false"
			else
				settings[nb]="true"
			end
		end
	}
end

def filter_list
	@dir=File.dirname(__FILE__)
	settings=IO.readlines(File.join(@dir,'Matt_Normalize/default.txt'))
	settings.each{|l|l.strip!}
	nb=0
	l=[]
	while nb < settings.length
		if settings[nb]=="true"
			case nb
				when 0
				l.push Sketchup::Drawingelement
				when 1
				l.push Sketchup::Edge
				when 2
				l.push Sketchup::Face
				when 3
				l.push Sketchup::Text
				when 4
				l.push Sketchup::ComponentInstance
				when 5
				l.push Sketchup::Group
				when 6
				l.push Sketchup::ConstructionLine
				when 7
				l.push Sketchup::ConstructionPoint
				when 8
				l.push Sketchup::SectionPlane
				when 9
				@flag=true
				
			end
		end
		nb+=1
	end
	return l
end

unless file_loaded?(__FILE__)
	file_loaded(__FILE__)
	$submenu ? (organizer = $submenu) : (organizer = UI.menu("Plugins"))
	menu = organizer.add_submenu('Matt_Normalize')
	menu.add_item('All'){all}
	menu.add_item('Selection'){selection}
	menu.add_separator
	menu.add_item('Filter options'){wd}
end

end
end
